@@ -41,7 +41,7 @@
 #include "eog-window.h"
 #include "util.h"
 #include "zoom.h"
-#include "eog-file-selection.h"
+#include "eog-file-chooser.h"
 #include "eog-preferences.h"
 #include "eog-statusbar.h"
 #include "libeog-marshal.h"
@@ -53,14 +53,25 @@
 #include "eog-horizontal-splitter.h"
 #include "eog-info-view.h"
 #include "eog-full-screen.h"
-#include "eog-save-dialog-helper.h"
 #include "eog-image-save-info.h"
-#include "eog-hig-dialog.h"
 #include "eog-uri-converter.h"
+#include "eog-save-dialog-helper.h"
 #include "eog-save-as-dialog-helper.h"
 #include "eog-pixbuf-util.h"
 #include "eog-job-manager.h"
 
+#if HAVE_LCMS
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <lcms.h>
+#endif
+
+#include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprintui/gnome-print-dialog.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
+
+
 #ifdef G_OS_WIN32
 
 #define getgid() 0
@@ -428,7 +439,7 @@ verb_FileOpen_cb (GtkAction *action, gpointer user_data)
 	window = EOG_WINDOW (user_data);
 	priv = window->priv;
 
-	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_OPEN);
+	dlg = eog_file_chooser_new (GTK_FILE_CHOOSER_ACTION_OPEN);
 
 	gtk_widget_show_all (dlg);
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -457,7 +468,7 @@ verb_FolderOpen_cb (GtkAction *action, gpointer user_data)
 	window = EOG_WINDOW (user_data);
 	priv = window->priv;
 
-	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	dlg = eog_file_chooser_new (GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 
 	gtk_widget_show_all (dlg);
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -498,8 +509,11 @@ verb_HelpAbout_cb (GtkAction *action, gpointer data)
 	GdkPixbuf *pixbuf;
 	
 	static const char *authors[] = {
-		"Jens Finke <jens@triq.net>",
+		"Tim Gerla <tim+gnomebugs@gerla.net> (maintainer)",
 		"",
+		"Philip Van Hoof <pvanhoof@gnome.org>",
+                "Paolo Borelli <pborelli@katamail.com>",
+		"Jens Finke <jens@triq.net>",
 		"Martin Baulig <martin@home-of-linux.org>",
 		"Arik Devens <arik@gnome.org>",
 		"Michael Meeks <mmeeks@gnu.org>",
@@ -681,7 +695,7 @@ verb_FullScreen_cb (GtkAction *action, gpointer data)
 	}
 
 	if (list != NULL) {
-		fs = eog_full_screen_new (list, start_image);
+		fs = eog_full_screen_new (GTK_WINDOW (data), list, start_image);
 		g_signal_connect (G_OBJECT (fs), "hide", G_CALLBACK (slideshow_hide_cb), EOG_WINDOW (data));
 
 		gtk_widget_show_all (fs);
@@ -857,17 +871,18 @@ save_error (SaveErrorData *edata)
 		char *str;
 
 		str = eog_image_get_uri_for_display (edata->image); 
-
 		header = g_strdup_printf (_("Overwrite file %s?"), str);
 		detail = _("File exists. Do you want to overwrite it?");
-
-		dlg = eog_hig_dialog_new (GTK_WINDOW (edata->window), 
-					  GTK_STOCK_DIALOG_ERROR,
-					  header, detail,
-					  TRUE);
-
 		g_free (str);
 
+		dlg = gtk_message_dialog_new (GTK_WINDOW (edata->window),
+					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_NONE,
+					      header);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+							  detail);
+
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Skip"), EOG_SAVE_RESPONSE_SKIP);
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Overwrite"), EOG_SAVE_RESPONSE_OVERWRITE);
 		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, EOG_SAVE_RESPONSE_CANCEL);
@@ -875,11 +890,14 @@ save_error (SaveErrorData *edata)
 	}
 	else {
 		header = g_strdup_printf (_("Error on saving %s."), eog_image_get_caption (edata->image));
-		
-		dlg = eog_hig_dialog_new (GTK_WINDOW (edata->window), 
-					  GTK_STOCK_DIALOG_ERROR,
-					  header, detail,
-					  TRUE);
+
+		dlg = gtk_message_dialog_new (GTK_WINDOW (edata->window),
+					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_NONE,
+					      header);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+							  detail);
 	
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Skip"), EOG_SAVE_RESPONSE_SKIP);
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Retry"), EOG_SAVE_RESPONSE_RETRY);
@@ -1385,7 +1403,7 @@ save_as_uri_selection_dialog (EogWindow *window, EogImage *image, char **uri, Gd
 		return;
 	}
 
-	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SAVE);
+	dlg = eog_file_chooser_new (GTK_FILE_CHOOSER_ACTION_SAVE);
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg),
 						 folder_uri);
 	while (!success) {
@@ -1401,7 +1419,7 @@ save_as_uri_selection_dialog (EogWindow *window, EogImage *image, char **uri, Gd
 
 		if (response == GTK_RESPONSE_OK) {
 			/* try to determine uri and image format */
-			*format = eog_file_selection_get_format (EOG_FILE_SELECTION (dlg));
+			*format = eog_file_chooser_get_format (EOG_FILE_CHOOSER (dlg));
 			*uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dlg));
 
 			if (*format == NULL && *uri != NULL) {
@@ -1427,11 +1445,14 @@ save_as_uri_selection_dialog (EogWindow *window, EogImage *image, char **uri, Gd
 			g_free (uesc_uri);
 			g_free (short_name);
 
-			err_dlg = eog_hig_dialog_new (GTK_WINDOW (window),
-						      GTK_STOCK_DIALOG_ERROR,
-						      header, detail,
-						      TRUE);
-			
+			err_dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+							  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+							  GTK_MESSAGE_ERROR,
+							  GTK_BUTTONS_NONE,
+							  header);
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dlg),
+							  	  detail);
+
 			gtk_dialog_add_button (GTK_DIALOG (err_dlg), _("Retry"), GTK_RESPONSE_OK);
 			gtk_dialog_add_button (GTK_DIALOG (err_dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 			gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
@@ -1551,16 +1572,18 @@ save_as_uri_converter_dialog (EogWindow *window, GList *images)
 		success = eog_uri_converter_check (conv, images, &error);
 
 		if (!success) {
-			GtkWidget *error_dlg;
+			GtkWidget *err_dlg;
 
-			error_dlg = eog_hig_dialog_new (GTK_WINDOW (window),
-							GTK_STOCK_DIALOG_ERROR,
-							_("Error on saving images."),
-							error->message,
-							TRUE);
-			gtk_dialog_add_button (GTK_DIALOG (error_dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
-			gtk_dialog_run (GTK_DIALOG (error_dlg));
-			gtk_widget_destroy (error_dlg);
+			err_dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+							  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+							  GTK_MESSAGE_ERROR,
+							  GTK_BUTTONS_OK,
+							  _("Error on saving images."));
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dlg),
+							  	  error->message);
+
+			gtk_dialog_run (GTK_DIALOG (err_dlg));
+			gtk_widget_destroy (err_dlg);
 
 			g_object_unref (conv);
 			conv = NULL;
@@ -1623,6 +1646,66 @@ save_as_multiple_images (EogWindow *window, GList *images)
 	g_object_unref (G_OBJECT (job));
 }
 
+static void
+verb_Print_cb (GtkAction *action, gpointer data)
+{
+	EogWindow *window = EOG_WINDOW (data);
+	EogWindowPrivate *priv = window->priv;	
+	EogImage *image = eog_wrap_list_get_first_selected_image (EOG_WRAP_LIST (priv->wraplist));
+
+	GnomePrintConfig* config = gnome_print_config_default();
+	GnomePrintContext *pc;
+	GnomePrintJob *gpj = gnome_print_job_new(config);
+	gint do_preview = 0 /*, copies = 1, collate = 0*/;
+	gdouble width, height;
+	GnomePrintDialog *gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(gpj, 
+		(const guchar*)_("Print"), GNOME_PRINT_DIALOG_COPIES));
+		
+	switch (gtk_dialog_run(GTK_DIALOG(gpd)))
+	{
+	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
+		do_preview = 0;
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
+		do_preview = 1;
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_CANCEL:
+		gnome_print_config_unref(config);
+		gnome_print_job_close (gpj);
+		gtk_widget_destroy (GTK_WIDGET(gpd));
+		return;
+	}
+	
+	gtk_widget_destroy(GTK_WIDGET(gpd));
+	
+	pc = gnome_print_job_get_context (gpj);
+	
+	gnome_print_beginpage(pc, (const guchar*)"1");
+	gnome_print_config_get_page_size(config, &width, &height);
+
+	gnome_print_gsave (pc);	
+	eog_image_print (image, pc, width, height);
+	gnome_print_grestore (pc);
+	
+	gnome_print_showpage(pc);
+	gnome_print_job_close (gpj);
+	
+	if (do_preview)
+	{
+		gtk_widget_show (gnome_print_job_preview_new 
+			(gpj, (const guchar*)_("Print Preview")));
+	}
+	else
+	{
+		gnome_print_job_print(gpj);
+	}
+	
+	gnome_print_context_close (pc);
+	gnome_print_config_unref(config);
+	
+
+}
+
 static void
 verb_SaveAs_cb (GtkAction *action, gpointer data)
 {
@@ -1811,12 +1894,15 @@ show_delete_confirm_dialog (EogWindow *window, int n_images)
 	header = g_strdup_printf (ngettext ("Do you really want to move %i image to trash?", 
 					    "Do you really want to move %i images to trash?", n_images), n_images);
 
-	dlg = eog_hig_dialog_new (GTK_WINDOW (window), GTK_STOCK_DIALOG_WARNING,
-				  header, NULL, TRUE);
+	dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+				      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				      GTK_MESSAGE_QUESTION,
+				      GTK_BUTTONS_NONE,
+				      header);
 	g_free (header);
 
-	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_DELETE, GTK_RESPONSE_OK);
 	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_DELETE, GTK_RESPONSE_OK);
 	gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
 	gtk_widget_show_all (dlg);
 
@@ -1921,17 +2007,19 @@ verb_Delete_cb (GtkAction *action, gpointer data)
 			GtkWidget *dlg;
 			
 			header = g_strdup_printf (_("Error on deleting image %s"), eog_image_get_caption (image));
-			
-			dlg = eog_hig_dialog_new (GTK_WINDOW (window), 
-						  GTK_STOCK_DIALOG_WARNING, header, 
-						  error->message, TRUE);
-			
-			gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
-			gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
-			gtk_widget_show_all (dlg);
-			
+
+			dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+						      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_ERROR,
+						      GTK_BUTTONS_OK,
+						      header);
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+								  error->message);
+
 			gtk_dialog_run (GTK_DIALOG (dlg));
 			gtk_widget_destroy (dlg);
+
+			g_free (header);
 		}
 	}
 
@@ -2591,13 +2679,20 @@ setup_initial_geometry (EogWindow *window)
 	int x, y;
 	GdkScreen *screen;
 	int screen_width, screen_height;
+	GdkRectangle monitor;
 	
 	g_assert (EOG_IS_WINDOW (window));
 
 	screen = gtk_window_get_screen (GTK_WINDOW (window));
 	g_assert (screen != NULL);
-	screen_width  = gdk_screen_get_width  (screen);
-	screen_height = gdk_screen_get_height (screen);
+
+	gtk_widget_realize (GTK_WIDGET (window));
+	gdk_screen_get_monitor_geometry (screen,
+			gdk_screen_get_monitor_at_window (screen,
+				GTK_WIDGET (window)->window),
+			&monitor);
+	screen_width  = monitor.width;
+	screen_height = monitor.height;
 
 	obtain_desired_size (window, screen_width, screen_height, &x11_flags,
 						 &x, &y, &width, &height);
@@ -2723,19 +2818,58 @@ show_error_dialog (EogWindow *window, char *header, GError *error)
 		detail = g_strdup_printf (_("Reason: %s"), error->message);
 	}
 
-	dlg = eog_hig_dialog_new (GTK_WINDOW (window), 
-				  GTK_STOCK_DIALOG_ERROR,
-				  header, detail,
-				  TRUE);
+	dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+				      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				      GTK_MESSAGE_ERROR,
+				      GTK_BUTTONS_OK,
+				      header);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+						  detail);
 
-	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
-	gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
 	g_signal_connect_swapped (G_OBJECT (dlg), "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  dlg);
 	gtk_widget_show_all (dlg);
 }
 
+#if HAVE_LCMS
+/* TODO: move into library */
+static cmsHPROFILE *
+get_screen_profile (EogWindow *window)
+{
+	GdkScreen *screen;
+	Display *dpy;
+	Atom icc_atom, type;
+	int format;
+	gulong nitems;
+	gulong bytes_after;
+	guchar *str;
+	int result;
+	cmsHPROFILE *profile;
+	
+	g_return_val_if_fail (window != NULL, NULL);
+	
+	screen = gtk_widget_get_screen (GTK_WIDGET (window));
+	dpy = GDK_DISPLAY_XDISPLAY (gdk_screen_get_display (screen));
+	icc_atom = gdk_x11_get_xatom_by_name_for_display (gdk_screen_get_display (screen), "_ICC_PROFILE");
+	
+	result = XGetWindowProperty (dpy, GDK_WINDOW_XID (gdk_screen_get_root_window (screen)),
+				     icc_atom, 0, G_MAXLONG,
+				     False, XA_CARDINAL, &type, &format, &nitems,
+				     &bytes_after, (guchar **)&str);
+	/* TODO: handle bytes_after != 0 */
+	
+	if (nitems) {
+		profile = cmsOpenProfileFromMem(str, nitems);
+		XFree (str);
+		return profile;
+	} else {
+		g_printerr("No profile, not correcting\n");
+		return NULL;
+	}
+}
+#endif
+
 static void 
 display_image_data (EogWindow *window, EogImage *image)
 {
@@ -2795,6 +2929,9 @@ job_image_load_finished (EogJob *job, gpointer data, GError *error)
 	if (eog_job_get_status (job) == EOG_JOB_STATUS_FINISHED) {
 		if (eog_job_get_success (job)) { 
 			/* successfull */
+#if HAVE_LCMS
+			eog_image_apply_display_profile (image, get_screen_profile (window));
+#endif
 			display_image_data (window, image);
 		}
 		else {  /* failed */
@@ -2963,6 +3100,7 @@ static const GtkToggleActionEntry toggle_entries_window[] = {
 static const GtkActionEntry action_entries_image[] = {
   { "FileSave", GTK_STOCK_SAVE, N_("_Save"), "<control>s", NULL, G_CALLBACK (verb_Save_cb) },
   { "FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As"), "<control><shift>s", NULL, G_CALLBACK (verb_SaveAs_cb) },
+  { "FilePrint", GTK_STOCK_PRINT, N_("Print"), "<control>p", NULL, G_CALLBACK (verb_Print_cb) },
 
   { "EditUndo", NULL, N_("_Undo"), "<control>z", NULL, G_CALLBACK (verb_Undo_cb) },
 
@@ -2979,8 +3117,11 @@ static const GtkActionEntry action_entries_image[] = {
   { "ViewSlideshow", NULL, N_("_Slideshow"), "F11", NULL, G_CALLBACK (verb_FullScreen_cb) },
   { "ViewZoomIn", GTK_STOCK_ZOOM_IN, N_("_Zoom In"), "<control>plus", NULL, G_CALLBACK (verb_ZoomIn_cb) },
   { "ViewZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "<control>minus", NULL, G_CALLBACK (verb_ZoomOut_cb) },
-  { "ViewZoomNormal", GTK_STOCK_ZOOM_100, N_("_Normal Size"), "<control>equal", NULL, G_CALLBACK (verb_ZoomNormal_cb) },
-  { "ViewZoomFit", GTK_STOCK_ZOOM_FIT, N_("Best _Fit"), NULL, NULL, G_CALLBACK (verb_ZoomFit_cb) }
+  { "ViewZoomNormal", GTK_STOCK_ZOOM_100, N_("_Normal Size"), "<control>0", NULL, G_CALLBACK (verb_ZoomNormal_cb) },
+  { "ViewZoomFit", GTK_STOCK_ZOOM_FIT, N_("Best _Fit"), NULL, NULL, G_CALLBACK (verb_ZoomFit_cb) },
+
+  /* accelerators */
+  { "ControlEqual", GTK_STOCK_ZOOM_IN, N_("_Zoom In"), "<control>equal", NULL, G_CALLBACK (verb_ZoomIn_cb) }
 };
 
 static const GtkToggleActionEntry toggle_entries_image[] = {
@@ -3471,15 +3612,16 @@ gboolean
 eog_window_open (EogWindow *window, EogImageList *model, GError **error)
 {
 	EogWindowPrivate *priv;
+	int i;
 
 	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
-	
+
 	priv = window->priv;
 
 #ifdef DEBUG
 	g_print ("EogWindow.c: eog_window_open\n");
 #endif
-	
+
 	/* attach image list */
 	if (priv->image_list != NULL) {
 		g_signal_handler_disconnect (G_OBJECT (priv->image_list), priv->sig_id_list_prepared);
@@ -3490,6 +3632,15 @@ eog_window_open (EogWindow *window, EogImageList *model, GError **error)
 	if (model != NULL) {
 		g_object_ref (model);
 		priv->image_list = model;
+
+#if HAVE_LCMS
+		/* Colour-correct the images */
+		for (i = 0; i < eog_image_list_length (model); i++) {
+			eog_image_apply_display_profile (eog_image_list_get_img_by_pos (model, i),
+							 get_screen_profile (window));
+		}
+#endif
+
 	}
 
 	/* update ui */
